// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: pi.cc,v 1.5 2003/03/03 21:57:00 jewillco Exp $
//
// This program is similar to the MPI example on page 28 of "Using MPI",
// by William Gropp, Ewing Lusk, and Anthony Skjellum.  It approximates
// the value of PI by integrating 4/(1.0 + x*x) over 0 <= x <= 1.  Each
// processor does part of the calculation.  The partial sums are then
// reduced to a single value that approximates PI.  By increasing the
// number of intervals, better approximations can gotten.

#include <iostream>
#include <math.h>
#include "oompi.h"

using namespace std;

int
main(int argc, char *argv[])
{
  int n, rank, size, i;
  double PI25DT = 3.141592653589793238462643;
  double mypi, pi, h, sum, x;

  OOMPI_COMM_WORLD.Init(argc, argv);
  size = OOMPI_COMM_WORLD.Size();
  rank = OOMPI_COMM_WORLD.Rank();

  while(1) {
    if (rank == 0) {
      cout << "Enter the number of intervals: (0 quits)" << endl;
      cin >> n;
    }

    // Broadcast the number of intervals
    OOMPI_COMM_WORLD[0].Bcast(n);

    if (n==0)
      break;
    else {
      h = 1.0 / (double) n;
      sum = 0.0;
      for (i = rank + 1; i <= n; i += size) {
	x = h * ((double)i - 0.5);
	sum += (4.0 / (1.0 + x*x));
      }
      mypi = h * sum;

      // Combine all the partial results
      OOMPI_COMM_WORLD[0].Reduce(mypi, pi, OOMPI_SUM);

      if (rank == 0)
	cout << "pi is approximately " << pi << ", Error is " << 
	  fabs(pi - PI25DT) << endl;
    }
  }

  OOMPI_COMM_WORLD.Finalize();
}

