// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: ring1.cc,v 1.4 2003/03/03 21:57:00 jewillco Exp $
//
// This program will send a message around a logical ring of processes.
// Each process sends the message to the process with the next highest
// rank.  The last process send to the 0 process.  The message that is
// being passed around is simply an integer.  

// Notice that each process is referred to (implicitly) as ports on the
// communicator with the bracket operator.  For example:
// OOMPI_COMM_WORLD[to].Send(i) send a message to the process which is at
// port 'to' on the OOMPI_COMM_WORLD communicator.


#include <iostream> 
#include "oompi.h"

using namespace std;

int
main(int argc, char *argv[])
{
  int count = 5;
  OOMPI_COMM_WORLD.Init(argc, argv);
  int i = 0;
  int msg = 123;

  int rank = OOMPI_COMM_WORLD.Rank();
  int size = OOMPI_COMM_WORLD.Size();
  int to   = (rank + 1) % size;
  int from = (size + rank - 1) % size;
  
  cout << "I am node " << rank << " of " << size << endl;
  cout << "Sending to " << to << " and receiving from " << from << endl;
  
  if (rank == size - 1)
    OOMPI_COMM_WORLD[to].Send(msg);
  
  for (i = 0; i < count; i++) {
    OOMPI_COMM_WORLD[from].Recv(msg);
    cout << "Node " << rank << " received " << msg << endl;
    OOMPI_COMM_WORLD[to].Send(msg);
  }

  if (rank == 0) {
    OOMPI_COMM_WORLD[from].Recv(msg);
    cout << "Node " << rank << " received " << msg << endl;
  }

  cout << "All done!" << endl;
  OOMPI_COMM_WORLD.Finalize();
  
  return 0;
}
