// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: environment_test.cc,v 1.8 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Environment functionality
// 

#include <string.h>

#include "oompi.h"
#include "oompi_test.h"
#if OOMPI_HAVE_MPP_RASTREAM_H
#include <mpp/rastream.h>
#endif


//
// OOMPI_Environment_test
//
void
OOMPI_Environment_test()
{
  bool passed;

  Testing("OOMPI_Environment");

  // Default constructor

  Testing("OOMPI_Environment::Default constructor");
  OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_EXCEPTION);
  passed = false;
#if OOMPI_HAVE_EXCEPTIONS
  try {
    OOMPI_Environment barf;
  }
  catch (OOMPI_Error &err) {
    passed = true;
  }
#else
  OOMPI_Environment barf1;
  if (OOMPI_errno != OOMPI_SUCCESS)
    passed = true;
#endif
  Test(passed);

  // Copy constructor

  Testing("OOMPI_Environment::Copy constructor");
  passed = false;
#if OOMPI_HAVE_EXCEPTIONS
  try {
    OOMPI_Environment barf(OOMPI_ENV);
  }
  catch (OOMPI_Error &err) {
    passed = true;
  }
#else
  OOMPI_Environment barf2(OOMPI_ENV);
  if (OOMPI_errno != OOMPI_SUCCESS)
    passed = true;
#endif
  Test(passed);

  // Copy constructor

  Testing("OOMPI_Environment::operator=");
  passed = false;
#if OOMPI_HAVE_EXCEPTIONS
  try {
    OOMPI_Environment barf = OOMPI_ENV;
  }
  catch (OOMPI_Error &err) {
    passed = true;
  }
#else 
  OOMPI_Environment barf3 = OOMPI_ENV;
  if (OOMPI_errno != OOMPI_SUCCESS)
    passed = true;
#endif

  // Reset error handler to ERRORS_ARE_FATAL
  OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_ARE_FATAL);

  Test(passed);

  // Buffer access

  Testing("OOMPI_Environment::Buffer_attach/detach()");
  {
#if OOMPI_HAVE_MPP_RASTREAM_H
    int oldstr = get_d_stream();
    set_d_stream(0);
#endif
    OOMPI_ENV.Buffer_attach(512);
    if (OOMPI_ENV.Buffer_detach() != 512)
      Fail("Detach size != attach size (non-alloc)");
    char *barf = new char[1024];
    OOMPI_ENV.Buffer_attach(barf, 1024);
    if (OOMPI_ENV.Buffer_detach() != 1024)
      Fail("Detach size != attach size (alloc)");
    delete[] barf;
#if OOMPI_HAVE_MPP_RASTREAM_H
    set_d_stream(oldstr);
#endif
    Pass();
  }
  
  // Processor dependant stuff

  Testing("OOMPI_Environment::Get_processor_name()");
  int len;
  char* barf = OOMPI_ENV.Get_processor_name();
  if (barf == 0)
    Fail("No hostname returned -- is it set properly?");
  delete[] barf;
  barf = OOMPI_ENV.Get_processor_name(len);
  if (barf == 0 || len != (int) strlen(barf))
    Fail("No hostname returned / string length not equal");
  Pass();
  delete[] barf;

  // Wtick / Wtime

  Testing("OOMPI_Environment::Wtick()");
  Test(OOMPI_ENV.Wtick() > 0);

  Testing("OOMPI_Environment::Wtime()");
  Test(OOMPI_ENV.Wtime() && 1);

  // Pcontrol

  Testing("OOMPI_Environment::Pcontrol()");
  OOMPI_ENV.Pcontrol(1);
  OOMPI_ENV.Pcontrol(2);
  OOMPI_ENV.Pcontrol(0);
  Pass();

  // Free constants
  // OOMPI_TAG_UB
  
  Testing("OOMPI_Environment::Free constants");
  {
    Testing("OOMPI_RESERVED_TAGS");
    Test(OOMPI_RESERVED_TAGS > 0);

    Testing("OOMPI_TAG_UB");
    Test(OOMPI_TAG_UB >= (32767 - OOMPI_RESERVED_TAGS));
    
    Testing("OOMPI_HOST");
    Test(OOMPI_HOST >= 0 || OOMPI_HOST == OOMPI_PROC_NULL);

    Testing("OOMPI_IO");
    {
      Test((OOMPI_IO >= 0 && OOMPI_IO < comm_size) || 
	   OOMPI_IO == OOMPI_ANY_SOURCE || 
	   OOMPI_IO == OOMPI_PROC_NULL);
    }

    Testing("OOMPI_WTIME_IS_GLOBAL");
    Test(OOMPI_WTIME_IS_GLOBAL == true || OOMPI_WTIME_IS_GLOBAL == false);
  }
  Pass(); // Free constants

  Pass(); // OOMPI_Environment
}
