// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: inter_comm_test.cc,v 1.4 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Inter_comm functionality
//

#include "oompi.h"
#include "oompi_test.h"


void 
OOMPI_Inter_comm_test(void)
{
  Testing ("OOMPI_Inter_Comm");

  bool am_even = (bool) ((my_rank % 2) == 0);
  OOMPI_Intra_comm oompi_intra = OOMPI_COMM_WORLD.Split(am_even);

  Testing("OOMPI_Inter_comm::Constructors");

  Testing("OOMPI_Inter_comm::Default constructor");
  {
    OOMPI_Inter_comm A;
    Test (A.Get_mpi() == MPI_COMM_NULL);
  }

  Testing("OOMPI_Inter_comm::Main constructor");
  MPI_Comm mpi_inter;
  int compare, remote_leader = am_even;

  MPI_Intercomm_create(oompi_intra.Get_mpi(), 0, MPI_COMM_WORLD, 
		       remote_leader, 7, &mpi_inter);
  OOMPI_Inter_comm B(oompi_intra, 0, OOMPI_COMM_WORLD, remote_leader, 8);
  MPI_Comm_compare(mpi_inter, B.Get_mpi(), &compare);
  Test(compare == MPI_CONGRUENT);

  Testing("OOMPI_Inter_comm::MPI constructor");
  {
    // Dup it so that I can use mpi_inter down below
    // mpi_inter2 is MPI_Comm_free()'ed at the end of this scope
    MPI_Comm mpi_inter2;
    MPI_Comm_dup(mpi_inter, &mpi_inter2);

    {
      OOMPI_Inter_comm B2(mpi_inter2);
      MPI_Comm_compare(B2.Get_mpi(), B.Get_mpi(), &compare);
    }
    MPI_Comm_free(&mpi_inter2);
    Test(compare == MPI_CONGRUENT);
  }

  OOMPI_Inter_comm C;
  Testing("OOMPI_Inter_comm::operator=()");
  {
    C = B;
    Test (B.Get_mpi() == C.Get_mpi());
  }

  Testing("OOMPI_Inter_comm::Copy constructor");
  {
    OOMPI_Inter_comm D(B);
    Test (B.Get_mpi() == D.Get_mpi());
  }

  Pass(); // Constructors

  Testing("OOMPI_Inter_comm::Test_inter()");
  Test(B.Test_inter());
  
  Testing("OOMPI_Inter_comm::Dup()");
  {
    C = B.Dup();
    Test(B.Get_mpi() != C.Get_mpi());
  }

  Testing("OOMPI_Inter_comm::Remote_group()");
  {
    OOMPI_Group G1 = B.Remote_group();
    MPI_Group remote_group;
    MPI_Comm_remote_group(mpi_inter, &remote_group);
    MPI_Group_compare(G1.Get_mpi(), remote_group, &compare);
    Test(compare == MPI_IDENT);
    MPI_Group_free(&remote_group);
  }

  Testing("OOMPI_Inter_comm::Remote_size()");
  {
    int remote_size;
    MPI_Comm_remote_size(mpi_inter, &remote_size);
    Test(B.Remote_size() == remote_size);
  }

  Testing("OOMPI_Inter_comm::Merge()");
  {
    OOMPI_Intra_comm M;
    M = B.Merge(am_even);
    MPI_Comm merged;
    MPI_Intercomm_merge(B.Get_mpi(), am_even, &merged);
    MPI_Comm_compare(merged, M.Get_mpi(), &compare);
    Test(compare == MPI_CONGRUENT);
    MPI_Comm_free(&merged);
  }

  Testing("OOMPI_Intracomm::Intercomm_create()");
  {
    OOMPI_Inter_comm oompi_inter2 = 
      oompi_intra.Intercomm_create(0, OOMPI_COMM_WORLD, remote_leader, 7);

    MPI_Comm mpi_inter2;
    MPI_Intercomm_create(oompi_intra.Get_mpi(), 0, MPI_COMM_WORLD, 
			 remote_leader, 8, &mpi_inter2);

    MPI_Comm_compare(mpi_inter2, oompi_inter2.Get_mpi(), &compare);
    Test(compare == MPI_CONGRUENT);
    MPI_Comm_free(&mpi_inter2);
  }    

  Testing("OOMPI_Port::Intercomm_create()");
  {
    OOMPI_Inter_comm oompi_inter3 = 
      oompi_intra[0].Intercomm_create(OOMPI_COMM_WORLD, remote_leader, 7);

    MPI_Comm mpi_inter3;
    MPI_Intercomm_create(oompi_intra.Get_mpi(), 0, MPI_COMM_WORLD, 
			 remote_leader, 8, &mpi_inter3);

    MPI_Comm_compare(mpi_inter3, oompi_inter3.Get_mpi(), &compare);
    Test(compare == MPI_CONGRUENT);
    MPI_Comm_free(&mpi_inter3);
  }

  // Clean up mpi_inter
 
  MPI_Comm_free(&mpi_inter);

  Pass(); // OOMPI_Inter_comm
}
