// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: message_test.cc,v 1.7 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Message functionality
//

#include "oompi.h"
#include "oompi_test.h"


//
// We must split up the message test to prevent optimizers from going nuts
//

static void msg_test1();
static void msg_test2();
static void msg_test3();
static void msg_test4();


//
// Helper functions
// Test promotion aspects
//
static inline void
Test_msg_param(OOMPI_Message a, void *top, MPI_Datatype realtype)
{
  if (!(a.Get_type() == realtype && a.Get_top() == top))
    Fail("Parameter message mismatch (single)");
}


static inline void
Test_msg_param(OOMPI_Message a, MPI_Datatype realtype, int realtag)
{
  if (!(a.Get_type() == realtype && a.Get_top() && a.Get_tag() == realtag))
    Fail("Parameter message type/tag mismatch");
}


//
// OOMPI_Message_test
//
void
OOMPI_Message_test()
{
  msg_test1();
  msg_test2();
  msg_test3();
  msg_test4();
}


//
// These all used to be part of the same routine, so when it was
// split, it was a heck of a lot simpler to just make them static
// globals.
//

static int i;
static char char_d[100];
static short short_d[100];
static int int_d[100];
static long long_d[100];
static unsigned char uchar_d[100];
static unsigned short ushort_d[100];
static unsigned u_d[100];
static double double_d[100];
static unsigned long ulong_d[100];
static float float_d[100];
#if OOMPI_HAVE_LONG_LONG_INT
static long long int llint_d[100];
#endif
#if OOMPI_HAVE_LONG_DOUBLE
static long double ldouble_d[100];
#endif
#if OOMPI_HAVE_ANSI_COMPLEX
static float_complex fcomplex_d[100];
static double_complex dcomplex_d[100];
static long_double_complex ldcomplex_d[100];
#endif



void
msg_test1()
{
  Testing("OOMPI_Message");

  // MPI Constructor

  Testing("OOMPI_Message::MPI constructor");

  OOMPI_Message mpi_msg1(OOMPI_INT, int_d);
  OOMPI_Message mpi_msg2(OOMPI_CHAR, char_d, 10);
  OOMPI_Message mpi_msg3(OOMPI_DOUBLE, double_d, 32, 1234);
  Midtest(mpi_msg1.Get_type() == MPI_INT &&
	  mpi_msg2.Get_type() == MPI_CHAR &&
	  mpi_msg3.Get_type() == MPI_DOUBLE, 
	  "Datatype mismatch in message");

  Midtest(mpi_msg1.Get_top() == int_d &&
	  mpi_msg2.Get_top() == char_d &&
	  mpi_msg3.Get_top() == double_d,
	  "Data pointer mismatch in message");

  Midtest(mpi_msg1.Get_count() == 1 &&
	  mpi_msg2.Get_count() == 10 &&
	  mpi_msg3.Get_count() == 32,
	  "Count mismatch in message");

  Midtest(mpi_msg1.Get_tag() == OOMPI_INT_TAG &&
	  mpi_msg2.Get_tag() == OOMPI_CHAR_TAG &&
	  mpi_msg3.Get_tag() == 1234,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::char constructors");
  Test_msg_param(char_d[0], &char_d[0], MPI_CHAR);
  Test_msg_param(char_d[0], MPI_CHAR, OOMPI_CHAR_TAG);
  OOMPI_Message char_msg1(char_d[0]);
  OOMPI_Message char_msg2(char_d[0], 1234);
  OOMPI_Message char_msg3(char_d, 100, 1111);
  Midtest(char_msg1.Get_type() == MPI_CHAR &&
	  char_msg2.Get_type() == MPI_CHAR &&
	  char_msg3.Get_type() == MPI_CHAR,
	  "Type mismatch in message");
  
  Midtest(char_msg1.Get_count() == 1 &&
	  char_msg2.Get_count() == 1 &&
	  char_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(char_msg1.Get_top() == &char_d[0] &&
	  char_msg2.Get_top() == &char_d[0] &&
	  char_msg3.Get_top() == &char_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(char_msg1.Get_tag() == OOMPI_CHAR_TAG &&
	  char_msg2.Get_tag() == 1234 &&
	  char_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::short constructors");
  Test_msg_param(short_d[0], &short_d[0], MPI_SHORT);
  Test_msg_param(short_d[0], MPI_SHORT, OOMPI_SHORT_TAG);
  OOMPI_Message short_msg1(short_d[0]);
  OOMPI_Message short_msg2(short_d[0], 1234);
  OOMPI_Message short_msg3(short_d, 100, 1111);
  Midtest(short_msg1.Get_type() == MPI_SHORT &&
	  short_msg2.Get_type() == MPI_SHORT &&
	  short_msg3.Get_type() == MPI_SHORT,
	  "Type mismatch in message");
  
  Midtest(short_msg1.Get_count() == 1 &&
	  short_msg2.Get_count() == 1 &&
	  short_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(short_msg1.Get_top() == &short_d[0] &&
	  short_msg2.Get_top() == &short_d[0] &&
	  short_msg3.Get_top() == &short_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(short_msg1.Get_tag() == OOMPI_SHORT_TAG &&
	  short_msg2.Get_tag() == 1234 &&
	  short_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::int constructors");
  Test_msg_param(int_d[0], &int_d[0], MPI_INT);
  Test_msg_param(int_d[0], MPI_INT, OOMPI_INT_TAG);
  OOMPI_Message int_msg1(int_d[0]);
  OOMPI_Message int_msg2(int_d[0], 1234);
  OOMPI_Message int_msg3(int_d, 100, 1111);
  Midtest(int_msg1.Get_type() == MPI_INT &&
	  int_msg2.Get_type() == MPI_INT &&
	  int_msg3.Get_type() == MPI_INT,
	  "Type mismatch in message");
  
  Midtest(int_msg1.Get_count() == 1 &&
	  int_msg2.Get_count() == 1 &&
	  int_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(int_msg1.Get_top() == &int_d[0] &&
	  int_msg2.Get_top() == &int_d[0] &&
	  int_msg3.Get_top() == &int_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(int_msg1.Get_tag() == OOMPI_INT_TAG &&
	  int_msg2.Get_tag() == 1234 &&
	  int_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::long constructors");
  Test_msg_param(long_d[0], &long_d[0], MPI_LONG);
  Test_msg_param(long_d[0], MPI_LONG, OOMPI_LONG_TAG);
  OOMPI_Message long_msg1(long_d[0]);
  OOMPI_Message long_msg2(long_d[0], 1234);
  OOMPI_Message long_msg3(long_d, 100, 1111);
  Midtest(long_msg1.Get_type() == MPI_LONG &&
	  long_msg2.Get_type() == MPI_LONG &&
	  long_msg3.Get_type() == MPI_LONG,
	  "Type mismatch in message");
  
  Midtest(long_msg1.Get_count() == 1 &&
	  long_msg2.Get_count() == 1 &&
	  long_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(long_msg1.Get_top() == &long_d[0] &&
	  long_msg2.Get_top() == &long_d[0] &&
	  long_msg3.Get_top() == &long_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(long_msg1.Get_tag() == OOMPI_LONG_TAG &&
	  long_msg2.Get_tag() == 1234 &&
	  long_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::unsigned char constructors");
  Test_msg_param(uchar_d[0], &uchar_d[0], MPI_UNSIGNED_CHAR);
  Test_msg_param(uchar_d[0], MPI_UNSIGNED_CHAR, 
		 OOMPI_UNSIGNED_CHAR_TAG);
  OOMPI_Message uchar_msg1(uchar_d[0]);
  OOMPI_Message uchar_msg2(uchar_d[0], 1234);
  OOMPI_Message uchar_msg3(uchar_d, 100, 1111);
  Midtest(uchar_msg1.Get_type() == MPI_UNSIGNED_CHAR &&
	  uchar_msg2.Get_type() == MPI_UNSIGNED_CHAR &&
	  uchar_msg3.Get_type() == MPI_UNSIGNED_CHAR,
	  "Type mismatch in message");
  
  Midtest(uchar_msg1.Get_count() == 1 &&
	  uchar_msg2.Get_count() == 1 &&
	  uchar_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(uchar_msg1.Get_top() == &uchar_d[0] &&
	  uchar_msg2.Get_top() == &uchar_d[0] &&
	  uchar_msg3.Get_top() == &uchar_d[0],
	  "Data poucharer mismatch in message");
  
  Midtest(uchar_msg1.Get_tag() == OOMPI_UNSIGNED_CHAR_TAG &&
	  uchar_msg2.Get_tag() == 1234 &&
	  uchar_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::unsigned short constructors");
  Test_msg_param(ushort_d[0], &ushort_d[0], MPI_UNSIGNED_SHORT);
  Test_msg_param(ushort_d[0], MPI_UNSIGNED_SHORT, 
		 OOMPI_UNSIGNED_SHORT_TAG);
  OOMPI_Message ushort_msg1(ushort_d[0]);
  OOMPI_Message ushort_msg2(ushort_d[0], 1234);
  OOMPI_Message ushort_msg3(ushort_d, 100, 1111);
  Midtest(ushort_msg1.Get_type() == MPI_UNSIGNED_SHORT &&
	  ushort_msg2.Get_type() == MPI_UNSIGNED_SHORT &&
	  ushort_msg3.Get_type() == MPI_UNSIGNED_SHORT,
	  "Type mismatch in message");
  
  Midtest(ushort_msg1.Get_count() == 1 &&
	  ushort_msg2.Get_count() == 1 &&
	  ushort_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(ushort_msg1.Get_top() == &ushort_d[0] &&
	  ushort_msg2.Get_top() == &ushort_d[0] &&
	  ushort_msg3.Get_top() == &ushort_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(ushort_msg1.Get_tag() == OOMPI_UNSIGNED_SHORT_TAG &&
	  ushort_msg2.Get_tag() == 1234 &&
	  ushort_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::unsigned constructors");
  Test_msg_param(u_d[0], &u_d[0], MPI_UNSIGNED);
  Test_msg_param(u_d[0], MPI_UNSIGNED, OOMPI_UNSIGNED_TAG);
  OOMPI_Message u_msg1(u_d[0]);
  OOMPI_Message u_msg2(u_d[0], 1234);
  OOMPI_Message u_msg3(u_d, 100, 1111);
  Midtest(u_msg1.Get_type() == MPI_UNSIGNED &&
	  u_msg2.Get_type() == MPI_UNSIGNED &&
	  u_msg3.Get_type() == MPI_UNSIGNED,
	  "Type mismatch in message");
  
  Midtest(u_msg1.Get_count() == 1 &&
	  u_msg2.Get_count() == 1 &&
	  u_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(u_msg1.Get_top() == &u_d[0] &&
	  u_msg2.Get_top() == &u_d[0] &&
	  u_msg3.Get_top() == &u_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(u_msg1.Get_tag() == OOMPI_UNSIGNED_TAG &&
	  u_msg2.Get_tag() == 1234 &&
	  u_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();
}

void
msg_test2()
{
  Testing("OOMPI_Message::unsigned long constructors");
  Test_msg_param(ulong_d[0], &ulong_d[0], MPI_UNSIGNED_LONG);
  Test_msg_param(ulong_d[0], MPI_UNSIGNED_LONG, 
		 OOMPI_UNSIGNED_LONG_TAG);
  OOMPI_Message ulong_msg1(ulong_d[0]);
  OOMPI_Message ulong_msg2(ulong_d[0], 1234);
  OOMPI_Message ulong_msg3(ulong_d, 100, 1111);
  Midtest(ulong_msg1.Get_type() == MPI_UNSIGNED_LONG &&
	  ulong_msg2.Get_type() == MPI_UNSIGNED_LONG &&
	  ulong_msg3.Get_type() == MPI_UNSIGNED_LONG,
	  "Type mismatch in message");
  
  Midtest(ulong_msg1.Get_count() == 1 &&
	  ulong_msg2.Get_count() == 1 &&
	  ulong_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(ulong_msg1.Get_top() == &ulong_d[0] &&
	  ulong_msg2.Get_top() == &ulong_d[0] &&
	  ulong_msg3.Get_top() == &ulong_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(ulong_msg1.Get_tag() == OOMPI_UNSIGNED_LONG_TAG &&
	  ulong_msg2.Get_tag() == 1234 &&
	  ulong_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::float constructors");
  Test_msg_param(float_d[0], &float_d[0], MPI_FLOAT);
  Test_msg_param(float_d[0], MPI_FLOAT, OOMPI_FLOAT_TAG);
  OOMPI_Message float_msg1(float_d[0]);
  OOMPI_Message float_msg2(float_d[0], 1234);
  OOMPI_Message float_msg3(float_d, 100, 1111);
  Midtest(float_msg1.Get_type() == MPI_FLOAT &&
	  float_msg2.Get_type() == MPI_FLOAT &&
	  float_msg3.Get_type() == MPI_FLOAT,
	  "Type mismatch in message");
  
  Midtest(float_msg1.Get_count() == 1 &&
	  float_msg2.Get_count() == 1 &&
	  float_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(float_msg1.Get_top() == &float_d[0] &&
	  float_msg2.Get_top() == &float_d[0] &&
	  float_msg3.Get_top() == &float_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(float_msg1.Get_tag() == OOMPI_FLOAT_TAG &&
	  float_msg2.Get_tag() == 1234 &&
	  float_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::double constructors");
  Test_msg_param(double_d[0], &double_d[0], MPI_DOUBLE);
  Test_msg_param(double_d[0], MPI_DOUBLE, OOMPI_DOUBLE_TAG);
  OOMPI_Message double_msg1(double_d[0]);
  OOMPI_Message double_msg2(double_d[0], 1234);
  OOMPI_Message double_msg3(double_d, 100, 1111);
  Midtest(double_msg1.Get_type() == MPI_DOUBLE &&
	  double_msg2.Get_type() == MPI_DOUBLE &&
	  double_msg3.Get_type() == MPI_DOUBLE,
	  "Type mismatch in message");
  
  Midtest(double_msg1.Get_count() == 1 &&
	  double_msg2.Get_count() == 1 &&
	  double_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(double_msg1.Get_top() == &double_d[0] &&
	  double_msg2.Get_top() == &double_d[0] &&
	  double_msg3.Get_top() == &double_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(double_msg1.Get_tag() == OOMPI_DOUBLE_TAG &&
	  double_msg2.Get_tag() == 1234 &&
	  double_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

#if OOMPI_HAVE_LONG_LONG_INT
  Testing("OOMPI_Message::long long int constructors");
  Test_msg_param(llint_d[0], &llint_d[0], MPI_LONG_LONG_INT);
  Test_msg_param(llint_d[0], MPI_LONG_LONGINT,
		 OOMPI_LONG_LONG_INT_TAG);
  OOMPI_Message llint_msg1(llint_d[0]);
  OOMPI_Message llint_msg2(llint_d[0], 1234);
  OOMPI_Message llint_msg3(llint_d, 100, 1111);
  Midtest(llint_msg1.Get_type() == MPI_LONG_LONG_INT &&
	  llint_msg2.Get_type() == MPI_LONG_LONG_INT &&
	  llint_msg3.Get_type() == MPI_LONG_LONG_INT,
	  "Type mismatch in message");
  
  Midtest(llint_msg1.Get_count() == 1 &&
	  llint_msg2.Get_count() == 1 &&
	  llint_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(llint_msg1.Get_top() == &llint_d[0] &&
	  llint_msg2.Get_top() == &llint_d[0] &&
	  llint_msg3.Get_top() == &llint_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(llint_msg1.Get_tag() == OOMPI_LONG_LONG_INT_TAG &&
	  llint_msg2.Get_tag() == 1234 &&
	  llint_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();
#endif

#if OOMPI_HAVE_LONG_DOUBLE
  Testing("OOMPI_Message::long double constructors");
  Test_msg_param(ldouble_d[0], &ldouble_d[0], MPI_LONG_DOUBLE);
  Test_msg_param(ldouble_d[0], MPI_LONG_DOUBLE,
		 OOMPI_LONG_DOUBLE_TAG);
  OOMPI_Message ldouble_msg1(ldouble_d[0]);
  OOMPI_Message ldouble_msg2(ldouble_d[0], 1234);
  OOMPI_Message ldouble_msg3(ldouble_d, 100, 1111);
  Midtest(ldouble_msg1.Get_type() == MPI_LONG_DOUBLE &&
	  ldouble_msg2.Get_type() == MPI_LONG_DOUBLE &&
	  ldouble_msg3.Get_type() == MPI_LONG_DOUBLE,
	  "Type mismatch in message");
  
  Midtest(ldouble_msg1.Get_count() == 1 &&
	  ldouble_msg2.Get_count() == 1 &&
	  ldouble_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(ldouble_msg1.Get_top() == &ldouble_d[0] &&
	  ldouble_msg2.Get_top() == &ldouble_d[0] &&
	  ldouble_msg3.Get_top() == &ldouble_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(ldouble_msg1.Get_tag() == OOMPI_LONG_DOUBLE_TAG &&
	  ldouble_msg2.Get_tag() == 1234 &&
	  ldouble_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();
#endif

#if OOMPI_HAVE_ANSI_COMPLEX
  Testing("OOMPI_Message::float complex constructors");
  Test_msg_param(fcomplex_d[0], &fcomplex_d[0], MPI_FLOAT_COMPLEX);
  Test_msg_param(fcomplex_d[0], MPI_FLOAT_COMPLEX, 
		 OOMPI_FLOAT_COMPLEX_TAG);
  OOMPI_Message fcomplex_msg1(fcomplex_d[0]);
  OOMPI_Message fcomplex_msg2(fcomplex_d[0], 1234);
  OOMPI_Message fcomplex_msg3(fcomplex_d, 100, 1111);
  Midtest(fcomplex_msg1.Get_type() == MPI_FLOAT_COMPLEX &&
	  fcomplex_msg2.Get_type() == MPI_FLOAT_COMPLEX &&
	  fcomplex_msg3.Get_type() == MPI_FLOAT_COMPLEX,
	  "Type mismatch in message");
  
  Midtest(fcomplex_msg1.Get_count() == 1 &&
	  fcomplex_msg2.Get_count() == 1 &&
	  fcomplex_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(fcomplex_msg1.Get_top() == &fcomplex_d[0] &&
	  fcomplex_msg2.Get_top() == &fcomplex_d[0] &&
	  fcomplex_msg3.Get_top() == &fcomplex_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(fcomplex_msg1.Get_tag() == OOMPI_FLOAT_COMPLEX_TAG &&
	  fcomplex_msg2.Get_tag() == 1234 &&
	  fcomplex_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::double complex constructors");
  Test_msg_param(dcomplex_d[0], &dcomplex_d[0], MPI_DOUBLE_COMPLEX);
  Test_msg_param(dcomplex_d[0], MPI_DOUBLE_COMPLEX, 
		 OOMPI_DOUBLE_COMPLEX_TAG);
  OOMPI_Message dcomplex_msg1(dcomplex_d[0]);
  OOMPI_Message dcomplex_msg2(dcomplex_d[0], 1234);
  OOMPI_Message dcomplex_msg3(dcomplex_d, 100, 1111);
  Midtest(dcomplex_msg1.Get_type() == MPI_DOUBLE_COMPLEX &&
	  dcomplex_msg2.Get_type() == MPI_DOUBLE_COMPLEX &&
	  dcomplex_msg3.Get_type() == MPI_DOUBLE_COMPLEX,
	  "Type mismatch in message");
  
  Midtest(dcomplex_msg1.Get_count() == 1 &&
	  dcomplex_msg2.Get_count() == 1 &&
	  dcomplex_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(dcomplex_msg1.Get_top() == &dcomplex_d[0] &&
	  dcomplex_msg2.Get_top() == &dcomplex_d[0] &&
	  dcomplex_msg3.Get_top() == &dcomplex_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(dcomplex_msg1.Get_tag() == OOMPI_DOUBLE_COMPLEX_TAG &&
	  dcomplex_msg2.Get_tag() == 1234 &&
	  dcomplex_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();

  Testing("OOMPI_Message::long double complex constructors");
  Test_msg_param(ldcomplex_d[0], &ldcomplex_d[0], MPI_LONG_DOUBLE_COMPLEX);
  Test_msg_param(ldcomplex_d[0], MPI_LONG_DOUBLE_COMPLEX,
		 OOMPI_LONG_DOUBLE_COMPLEX_TAG);
  OOMPI_Message ldcomplex_msg1(ldcomplex_d[0]);
  OOMPI_Message ldcomplex_msg2(ldcomplex_d[0], 1234);
  OOMPI_Message ldcomplex_msg3(ldcomplex_d, 100, 1111);
  Midtest(ldcomplex_msg1.Get_type() == MPI_LONG_DOUBLE_COMPLEX &&
	  ldcomplex_msg2.Get_type() == MPI_LONG_DOUBLE_COMPLEX &&
	  ldcomplex_msg3.Get_type() == MPI_LONG_DOUBLE_COMPLEX,
	  "Type mismatch in message");
  
  Midtest(ldcomplex_msg1.Get_count() == 1 &&
	  ldcomplex_msg2.Get_count() == 1 &&
	  ldcomplex_msg3.Get_count() == 100,
	  "Count mismatch in message");
  
  Midtest(ldcomplex_msg1.Get_top() == &ldcomplex_d[0] &&
	  ldcomplex_msg2.Get_top() == &ldcomplex_d[0] &&
	  ldcomplex_msg3.Get_top() == &ldcomplex_d[0],
	  "Data pointer mismatch in message");
  
  Midtest(ldcomplex_msg1.Get_tag() == OOMPI_LONG_DOUBLE_COMPLEX_TAG &&
	  ldcomplex_msg2.Get_tag() == 1234 &&
	  ldcomplex_msg3.Get_tag() == 1111,
	  "Tag mismatch in message");
  Pass();
#endif
}


void
msg_test3()
{
  // Pick up some tests that were previously postponed

  Testing("OOMPI_Comm");
  Testing("OOMPI_Comm::Pack_size()");
  char data_char = 'A'; 
  double data_double = 123.456;
  int size1, size2, size3;
  OOMPI_Message char_msg(OOMPI_CHAR, &data_char, 100);
  
  MPI_Pack_size(1, MPI_INT, MPI_COMM_WORLD, &size1);
  MPI_Pack_size(100, MPI_CHAR, MPI_COMM_WORLD, &size2);
  MPI_Pack_size(1234, MPI_DOUBLE, MPI_COMM_WORLD, &size3);

  Test(OOMPI_COMM_WORLD.Pack_size(OOMPI_CHAR, 100) == size2 &&
       OOMPI_COMM_WORLD.Pack_size(&data_char, 100) == size2 &&
       OOMPI_COMM_WORLD.Pack_size(char_msg) == size2 &&
       OOMPI_COMM_WORLD.Pack_size(&data_double, 1234) == size3);

  // Send to the next rank, receive from the previous rank (primitive
  // message ring)

  Testing("OOMPI_Comm::Sendrecv()");
  if (comm_size <= 1)
    Fail("Need to have multiple MPI processes (i.e. >1) to continue");

  int send[2] = { to, to + 1 };
  int recv[2] = { 0, 0 };

  OOMPI_COMM_WORLD.Sendrecv(send[0], to, 100, recv[0], from, 100);
  if (recv[0] != my_rank)
    Fail("Sendrecv() test 1 failed");

  recv[0]= 0; recv[1]= 0;
  OOMPI_COMM_WORLD.Sendrecv(send, 2, to, 100, recv, 2, from, 100);
  if (recv[0] != my_rank || recv[1] != my_rank + 1)
    Fail("Sendrecv() test 2 failed");
  
  // Now only use ranks 0 and 1 for other two forms of Sendrecv

  if (my_rank == 0) {
    send[0]= 1;
    send[1]= 2;
    recv[0]= 0; recv[1]= 0;
    to= from= 1;

    OOMPI_COMM_WORLD.Sendrecv(send, 2, to, 100, recv[0], from, 100);
    if (recv[0] != my_rank)
      Fail("Sendrecv() test 3 failed (rank 0)");

    recv[0]= 0; recv[1]= 0;
    OOMPI_COMM_WORLD.Sendrecv(send[0], to, 100, recv, 2, from, 100);
    if (recv[0] != my_rank || recv[1] != my_rank + 1)
      Fail("Sendrecv() test 4 failed (rank 0)");
  }
  else if (my_rank == 1) {
    send[0]= 0;
    send[1]= 1;
    recv[0]= 0; recv[1]= 0;
    to= from= 0;

    OOMPI_COMM_WORLD.Sendrecv(send[0], to, 100, recv, 2, from, 100);
    if (recv[0] != my_rank || recv[1] != my_rank + 1)
      Fail("Sendrecv() test 3 failed (rank 1)");

    recv[0]= 0; recv[1]= 0;
    OOMPI_COMM_WORLD.Sendrecv(send, 2, to, 100, recv[0], from, 100);
    if (recv[0] != my_rank)
      Fail("Sendrecv() test 4 failed (rank 1)");
  }
  Pass();

  // Same thing, but with sendrecv_replace
  // Primitive message ring

  Testing("OOMPI_Comm::Sendrecv_replace()");
  to = (my_rank + 1) % comm_size;
  from = (my_rank + comm_size - 1) % comm_size;
  send[0]= to;
  int *yow = new int[2];
  yow[0] = to;
  OOMPI_COMM_WORLD.Sendrecv_replace(send[0], to, 100, from, 100);
  if (send[0] != my_rank)
    Fail("Sendrecv_replace() test 1 failed");
  
  send[0]= to;
  send[1]= to + 1;
  OOMPI_COMM_WORLD.Sendrecv_replace(send, 2, to, 100, from, 100);
  if (send[0] != my_rank || send[1] != my_rank + 1)
    Fail("Sendrecv_replace() test 2 failed");
  Pass();
  delete[] yow;

  Pass(); // OOMPI_Comm
}

void
msg_test4()
{
  //
  // Big four tests
  // Copy constructor
  //
  
  Testing("OOMPI_Message::Copy constructor");
  OOMPI_Message mpi_msg3(OOMPI_DOUBLE, double_d, 32, 1234);
  OOMPI_Message yow1(mpi_msg3);
  Midtest(yow1.Get_type() == MPI_DOUBLE &&
	  yow1.Get_top() == double_d &&
	  yow1.Get_count() == 32 &&
	  yow1.Get_tag() == 1234,
	  "MPI constructor copy");
  
  OOMPI_Message char_msg3(char_d, 100, 1111);
  OOMPI_Message yow2(char_msg3);
  Midtest(yow2.Get_type() == MPI_CHAR &&
	  yow2.Get_top() == char_d &&
	  yow2.Get_count() == 100 &&
	  yow2.Get_tag() == 1111,
	  "Promotion constructor copy");
  Pass();
  
  Testing("OOMPI_Message::operator=()");
  yow1 = yow2;
  Midtest(yow1.Get_type() == MPI_CHAR &&
	  yow1.Get_top() == char_d &&
	  yow1.Get_count() == 100 &&
	  yow1.Get_tag() == 1111,
	  "Promotion constructor assignment");
  
  yow2= mpi_msg3;
  Midtest(yow2.Get_type() == MPI_DOUBLE &&
	  yow2.Get_top() == double_d &&
	  yow2.Get_count() == 32 &&
	  yow2.Get_tag() == 1234,
	  "MPI constructor copy");
  Pass();
  
  Testing("OOMPI_Message::Destructor");
  for (i= 0; i < 1; i++) {
    OOMPI_Message yow3(yow1);
    OOMPI_Message yow4(yow2);
  }
  Midtest(yow1.Get_type() == MPI_CHAR &&
	  yow1.Get_top() == char_d &&
	  yow1.Get_count() == 100 &&
	  yow1.Get_tag() == 1111,
	  "Shared promotion destructor");
  Midtest(yow2.Get_type() == MPI_DOUBLE &&
	  yow2.Get_top() == double_d &&
	  yow2.Get_count() == 32 &&
	  yow2.Get_tag() == 1234,
	  "Shared MPI destructor copy");
  Pass();

  Pass(); // OOMPI_Message
}
