// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: op_test.cc,v 1.5 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Op functionality
//

#include "oompi.h"
#include "oompi_test.h"

#if OOMPI_MPI_EXTERN_C
extern "C" {
#endif
  static void My_sum(void *invec, void *inoutvec, int *len, 
		     MPI_Datatype *type);
#if OOMPI_MPI_EXTERN_C
}
#endif

//
// Dummy op function
// Do a ((sum) + (len * size))
// Only works for int's.  Doh!
//
static void
My_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *type)
{
  int i;
  int *src= (int *) invec;
  int *dest= (int *) inoutvec;

  for (i = 0; i < *len; i++)
    if (*type != MPI_INT) 
      dest[i] = 0;
    else
      dest[i] += src[i] + (*len);
}


//
// OOMPI_Op_test
//
void
OOMPI_Op_test()
{
  int i;

  Testing("OOMPI_Op");

  // Default

  Testing("OOMPI_Op::Default constructor");
  OOMPI_Op op1;
  Test(op1.Is_null());

  // Main constructor

  Testing("OOMPI_Op::Main constructor");
  OOMPI_Op op2(My_sum, true);

  int send[2], recv[2], check[2];
  int len= 2;
  send[0]= my_rank;
  send[1]= my_rank * 3;
  MPI_Allreduce(send, recv, len, MPI_INT, op2.Get_mpi(), MPI_COMM_WORLD);

  check[0] = check[1] = 0;
  for (i= 1; i < comm_size; i++) {
    check[0] += i + len;
    check[1] += (i * 3) + len;
  }
  Test(check[0] == recv[0] && check[1] == recv[1]);

  // Is_null

  Testing("OOMPI_Op::Is_null()");
  Test(!op2.Is_null());

  // Big 4 tests
  // Assignment

  Testing("OOMPI_Op::operator=()");
  OOMPI_Op op3;
  op3 = op2;
  MPI_Allreduce(send, recv, len, MPI_INT, op3.Get_mpi(), MPI_COMM_WORLD);
  Test(check[0] == recv[0] && check[1] == recv[1]);

  // Copy constructor
  // Bogus logic to fool optimizing compilers

  Testing("OOMPI_Op::Copy constructor");
  for (i= 0; i < 1; i++) {
    OOMPI_Op op4(op2);
    MPI_Allreduce(send, recv, len, MPI_INT, op4.Get_mpi(), MPI_COMM_WORLD);
    Test(check[0] == recv[0] && check[1] == recv[1]);
  }

  // Destructor
  // (see if destruction of op3 trashed op2)

  Testing("OOMPI_Op::Destructor");
  MPI_Allreduce(send, recv, len, MPI_INT, op2.Get_mpi(), MPI_COMM_WORLD);
  Test(check[0] == recv[0] && check[1] == recv[1]);

  Pass();
}

