// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: signal.cc,v 1.10 2003/03/03 21:36:13 jewillco Exp $
//
// OOMPI test program
// Test *all* OOMPI functionality
//

#include <iostream>

#include <signal.h>

#include "oompi.h"
#include "oompi_test.h"


//
// Local variables
//

struct signal_map {
  int number;
  const char* name;
  struct sigaction* old_action;
};

struct signal_map block_me[] = {
  { SIGHUP, "Hangup", 0 },
  { SIGINT, "Interrupt",0  },
  { SIGQUIT, "Quit", 0 },
  { SIGTRAP, "IOT Trap", 0 },
  { SIGFPE, "Floating point exception", 0 },
  { SIGBUS, "Bus error", 0 },
  { SIGSEGV, "Segmentation fault", 0 },
  { SIGTERM, "Terminate", 0 },
  { SIGSTOP, "Stop", 0 },
  { SIGABRT, "Signal abort", 0 },
/* SIGIOT was not defined on Cygwin */
#ifdef SIGIOT
  { SIGIOT, "IOT instruction", 0 },
#endif
  { -1, (char *) -1, (struct sigaction *) -1 },
};


//
// Local functions
//

extern "C" {
#if OOMPI_BSD_SIGNAL
typedef void (*signal_handler)(...);
void handler(int sig, int code, struct sigcontext *scp, char *addr);
#elif OOMPI_SYSV_SIGNAL
typedef void (*signal_handler)(int);
void handler(int sig);
#endif
}


//
// init_signals
//
void
signal_init()
{
#if OOMPI_BSD_SIGNAL
  int i;
  struct sigaction n;

  n.sa_handler = (signal_handler) handler;
  sigemptyset(&n.sa_mask);
  n.sa_flags = 0;

  for (i = 0; block_me[i].number >= 0; i++) {
    // some compilers are very picky...
#if 0
    block_me[i].old_action= (struct sigaction *) new 
      char[sizeof(struct sigaction)];
#else
    block_me[i].old_action= new struct sigaction;
#endif
    sigaction(block_me[i].number, &n, block_me[i].old_action);
  }
#elif OOMPI_SYSV_SIGNAL
  int i;

  for (i = 0; block_me[i].number >= 0; i++)
    signal(block_me[i].number, handler);
#endif
}


//
// handler
//
#if OOMPI_BSD_SIGNAL
void 
handler(int sig, int code, struct sigcontext *scp, char *addr)
#elif OOMPI_SYSV_SIGNAL
void 
handler(int sig)
#endif
{
  int i;

  // So that g++ -Wall won't complain

#if OOMPI_BSD_SIGNAL
  i = code;
  i = (int) scp;
  i = (int) addr;
#endif

  // Find the name

  for (i = 0; block_me[i].number >= 0; i++)
    if (block_me[i].number == sig)
      break;
  
  std::cout << std::endl << std::endl << "OOMPI test suite (rank " 
	    << my_rank << "): ";
  if (block_me[i].number == -1)
    std::cout << "Unknown signal (" << sig << ") caught";
  else
    std::cout << block_me[i].name;
  std::cout << std::endl << "OOMPI test suite: aborting..." << std::endl;
  std::cout.flush();
    
  // Try to abort
  
  OOMPI_COMM_WORLD.Abort(OOMPI_ERR_OTHER);
  
  // Should never get here

  if (my_rank <= 0)
    std::cerr << "OOMPI test suite: terminated" << std::endl << std::endl;

  exit(OOMPI_ERR_OTHER);
}

